import { Button, Divider, Flex } from "antd";
import MyBanner from "shared/components/MyBanner";
import { MyContainer } from "shared/components/MyContainer";
import { PlusOutlined } from "@ant-design/icons";
import Search, { SearchProps } from "antd/es/input/Search";
import { useNavigate } from "react-router-dom";
import HeaderBar from "core/components/Header";
import {
  useCreateLessonMutation,
  useGetLessonsQuery,
} from "core/services/lessons";
import { Constants } from "core/utils/utils";
import MyEmpty from "shared/components/MyEmpty";
import MyErrorResult from "shared/components/MyResult";
import LessonPreviewCard from "shared/components/MyLessonPreviewCard";
import { useMessage } from "core/context/MessageContext";
import { useEffect } from "react";
import { useDispatch, useSelector } from "react-redux";
import { lessons, setLessons } from "./lessonsSlice";
import {
  addWebSocketReconnectListener,
  removeWebSocketReconnectListener,
} from "core/services/websocketService";
import MyCenteredSpin from "shared/components/MyCenteredSpin";

const CreateLessonButton: React.FC = () => {
  const navigate = useNavigate();
  const [createLesson, { isLoading }] = useCreateLessonMutation();
  const { success, error } = useMessage();

  const handleCreateLesson = async () => {
    try {
      const res = await createLesson({}).unwrap();

      if (res && res.Id) {
        success("Lesson created successfully  ");

        navigate(
          Constants.ROUTE_PATHS.LESSIONS.PAGE_EDITOR.replace(
            ":lessonId",
            res.Id
          )
        );
      }
    } catch (err) {
      console.error(err);
      error("Failed to create lesson");
    }
  };

  return (
    <Button
      icon={<PlusOutlined />}
      onClick={handleCreateLesson}
      loading={isLoading}
    >
      Create
    </Button>
  );
};

const LessonList: React.FC = () => {
  const dispatch = useDispatch();
  const dataLessons = useSelector(lessons);

  const { data, error, isLoading, refetch } = useGetLessonsQuery(undefined, {
    refetchOnMountOrArgChange: true,
  });

  useEffect(() => {
    if (!data) return;

    dispatch(setLessons(data));
  }, [data]);

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  if (isLoading) return <MyCenteredSpin height="240px" />;
  if (error) return <MyErrorResult />;

  if (!dataLessons || dataLessons.length === 0) return <MyEmpty />;

  const publishedItems = dataLessons.filter((item) => item.State === 1);
  const unpublishedItems = dataLessons.filter((item) => item.State === 2);

  return (
    <>
      {publishedItems.length > 0 && (
        <>
          {publishedItems.map((item, index) => (
            <LessonPreviewCard
              key={index}
              mode="view"
              lessonId={item.Id}
              loading={false}
              lessonSettings={{
                Title: item.Title,
                ThumbnailUrl: item.ThumbnailUrl,
              }}
            />
          ))}
        </>
      )}

      {unpublishedItems.length > 0 && (
        <>
          <Divider orientation="left" style={{ marginBottom: 0 }}>
            Unpublished
          </Divider>

          {unpublishedItems.map((item, index) => (
            <LessonPreviewCard
              key={index}
              mode="view"
              lessonId={item.Id}
              loading={false}
              lessonSettings={{
                Title: item.Title,
                ThumbnailUrl: item.ThumbnailUrl,
              }}
            />
          ))}
        </>
      )}
    </>
  );
};

export default function Lessons() {
  const onSearch: SearchProps["onSearch"] = (value, _e, info) =>
    console.log(info?.source, value);

  return (
    <>
      <MyBanner title="Lessons" headerBar={<HeaderBar />} />

      <MyContainer>
        <Flex justify="right" gap={16} style={{ paddingBottom: 16 }}>
          <CreateLessonButton />

          <Search
            placeholder="Search..."
            onSearch={onSearch}
            style={{ width: 300 }}
            allowClear
          />
        </Flex>

        <Flex vertical gap={16}>
          <LessonList />
        </Flex>
      </MyContainer>
    </>
  );
}
