import { Button, Flex, Form, Input, Modal } from "antd";
import HeaderBar from "../../core/components/Header";
import MyBanner from "../../shared/components/MyBanner";
import { SaveOutlined } from "@ant-design/icons";
import MyUpload from "shared/components/MyUpload";
import { Constants } from "core/utils/utils";
import ColorPicker from "antd/es/color-picker";
import {
  useGetOrganizationSettingsQuery,
  useIsSubdomainAvailableMutation,
  useUpdateOrganizationSettingsMutation,
  useUpdateSubdomainMutation,
} from "core/services/organization";
import MyErrorResult from "shared/components/MyResult";
import { useForm } from "antd/es/form/Form";
import { useEffect, useRef, useState } from "react";
import { AggregationColor } from "antd/es/color-picker/color";
import { useDispatch, useSelector } from "react-redux";
import {
  bannerUrl,
  logoUrl,
  setBannerUrl,
  setLogoUrl,
  setPrimaryColor,
} from "core/reducers/appSlice";
import MyMiddleCard from "shared/components/MyMiddleCard";
import { OrganizationSettings } from "core/types/organization";
import { useMessage } from "core/context/MessageContext";
import {
  addWebSocketReconnectListener,
  removeWebSocketReconnectListener,
} from "core/services/websocketService";

type GeneralFieldType = {
  primaryColor: string | AggregationColor;
  companyName: string;
};

export default function Settings() {
  const { data, error, isLoading, refetch } = useGetOrganizationSettingsQuery(
    undefined,
    {
      refetchOnMountOrArgChange: true,
    }
  );

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  if (error) return <MyErrorResult />;

  return (
    <>
      <MyBanner title="Settings" subtitle="MANAGE" headerBar={<HeaderBar />} />

      <GeneralCard data={data} isLoading={isLoading} />

      <MediaCard isLoading={isLoading} />

      <SubdomainCard data={data} isLoading={isLoading} />
    </>
  );
}

function GeneralCard({
  data,
  isLoading,
}: { data?: OrganizationSettings; isLoading?: boolean } = {}) {
  const [form] = useForm<GeneralFieldType>();
  const { success, error: errorMessage } = useMessage();

  const dispatch = useDispatch();
  const debounceRef = useRef<null | NodeJS.Timeout>(null);
  const currentPrimaryColor = useRef<string>();

  const [updateOrganizationSettings, { isLoading: isUpdateSettingsLoading }] =
    useUpdateOrganizationSettingsMutation();

  const handleSave = async (values: GeneralFieldType) => {
    const hexColor =
      typeof values.primaryColor === "string"
        ? values.primaryColor
        : values.primaryColor.toHexString().split("#")[1];

    try {
      await updateOrganizationSettings({
        primaryColor: hexColor,
        companyName: values.companyName,
      }).unwrap();

      currentPrimaryColor.current = hexColor;

      success("Settings updated successfully!");
    } catch (error) {
      console.error(error);
      errorMessage("Failed to update settings!");
    }
  };

  useEffect(() => {
    if (data) {
      form.setFieldsValue({
        primaryColor: data.PrimaryColor,
        companyName: data.CompanyName,
      });

      currentPrimaryColor.current = data.PrimaryColor;
    }
  }, [data]);

  useEffect(() => {
    return () => {
      dispatch(setPrimaryColor(currentPrimaryColor.current));

      if (debounceRef.current) {
        clearTimeout(debounceRef.current);
      }
    };
  }, []);

  return (
    <Form form={form} onFinish={handleSave} layout="vertical">
      <MyMiddleCard
        styles={{
          body: {
            padding: 16,
          },
        }}
        title="General"
        loading={isLoading}
        extra={
          <Button
            icon={<SaveOutlined />}
            type="text"
            shape="circle"
            size="large"
            htmlType="submit"
            loading={isUpdateSettingsLoading}
          />
        }
      >
        <Flex wrap gap={12}>
          <Form.Item<GeneralFieldType>
            name="primaryColor"
            label="Primary color"
          >
            <ColorPicker
              size="small"
              showText
              onChange={(color: AggregationColor) => {
                if (debounceRef.current) {
                  clearTimeout(debounceRef.current);
                }

                debounceRef.current = setTimeout(() => {
                  dispatch(setPrimaryColor(color.toHexString().split("#")[1]));
                }, 600);
              }}
            />
          </Form.Item>

          <Form.Item<GeneralFieldType> name="companyName" label="Company name">
            <Input />
          </Form.Item>
        </Flex>
      </MyMiddleCard>
    </Form>
  );
}

function MediaCard({
  isLoading,
}: { data?: OrganizationSettings; isLoading?: boolean } = {}) {
  const { success } = useMessage();
  const dispatch = useDispatch();

  const appLogoUrl = useSelector(logoUrl);
  const appBannerUrl = useSelector(bannerUrl);

  return (
    <Form layout="vertical">
      <MyMiddleCard title="Media" loading={isLoading}>
        <Form.Item label="Logo">
          <MyUpload
            action="/organization/file/logo"
            onChange={(info) => {
              if (info.file.status === "done" && info.file.response.Data) {
                dispatch(setLogoUrl(info.file.response.Data));

                success("Logo updated successfully!");
              }
            }}
            imgCropProps={{
              aspect: 1 / 1,
              children: <></>,
            }}
          >
            <img
              src={`${Constants.STATIC_CONTENT_ADDRESS}/${
                appLogoUrl || Constants.DEMO_LOGO_URL
              }`}
              alt="Company Logo"
              style={{
                width: 128,
                maxHeight: 128,
                padding: 4,
                borderRadius: 4,
                border: "1px solid #ddd",
              }}
            />
          </MyUpload>
        </Form.Item>

        <Form.Item label="Banner">
          <MyUpload
            action="/organization/file/banner"
            onChange={(info) => {
              if (info.file.status === "done" && info.file.response.Data) {
                dispatch(setBannerUrl(info.file.response.Data));

                success("Banner updated successfully!");
              }
            }}
            imgCropProps={{
              aspect: 22 / 9,
              children: <></>,
            }}
          >
            <img
              src={`${Constants.STATIC_CONTENT_ADDRESS}/${
                appBannerUrl || Constants.DEMO_BANNER_URL
              }`}
              alt="Banner"
              style={{
                width: "100%",
                height: 228,
                objectFit: "cover",
                padding: 4,
                borderRadius: 4,
                border: "1px solid #ddd",
              }}
            />
          </MyUpload>
        </Form.Item>
      </MyMiddleCard>
    </Form>
  );
}

const subdomainPattern = /^[a-zA-Z0-9-]+$/;

function SubdomainCard({
  data,
  isLoading,
}: { data?: OrganizationSettings; isLoading?: boolean } = {}) {
  const [form] = useForm();
  const { success, info } = useMessage();

  const [isModalOpen, setIsModalOpen] = useState(false);
  const [reqIsSubdomainAvailable] = useIsSubdomainAvailableMutation();
  const [reqUpdateSubdomain] = useUpdateSubdomainMutation();

  const validateSubdomain = async (rule: any, value: string) => {
    if (value) {
      if (
        value.length < Constants.GLOBALS.MIN_SUBDOMAIN_LENGTH ||
        value.length > Constants.GLOBALS.MAX_SUBDOMAIN_LENGTH ||
        !subdomainPattern.test(value)
      ) {
        return Promise.reject();
      }

      // Check if subdomain is current subdomain

      if (value === window.location.hostname.split(".")[0]) {
        return Promise.resolve();
      }

      try {
        const { data } = await reqIsSubdomainAvailable(value);

        if (!data.Available) {
          return Promise.reject("This subdomain is already taken!");
        }

        return Promise.resolve();
      } catch (error) {
        return Promise.reject("This subdomain is already taken!");
      }
    }

    return Promise.resolve();
  };

  useEffect(() => {
    if (data) {
      form.setFieldsValue({
        subdomain: data.Subdomain,
      });
    }
  }, [data]);

  return (
    <>
      <Modal
        title="Change Subdomain"
        open={isModalOpen}
        centered
        onCancel={() => setIsModalOpen(false)}
        okText="Change"
        onOk={async () => {
          try {
            await reqUpdateSubdomain(form.getFieldValue("subdomain"));

            success("Subdomain updated successfully!");
            info("You will be redirected to the new subdomain!");
            /*
            window.location.href = `https://${form.getFieldValue(
              "subdomain"
            )}.${window.location.hostname.split(".").slice(1).join(".")}`; */
          } catch (error) {
            console.error(error);
          }
        }}
      >
        <p>
          Changing your subdomain will make your organization available at the
          new subdomain. Please note that you will be logged out and redirected
          to the new subdomain. Also the old subdomain will be available for
          registration by other users.
        </p>
      </Modal>

      <Form form={form} layout="vertical" requiredMark={false}>
        <MyMiddleCard
          title="Subdomain"
          loading={isLoading}
          extra={
            <Button
              icon={<SaveOutlined />}
              type="text"
              shape="circle"
              size="large"
              onClick={() => {
                form
                  .validateFields()
                  .then(() => setIsModalOpen(true))
                  .catch(() => {
                    console.error("Validation failed!");
                  });
              }}
            />
          }
        >
          <Flex>
            <Form.Item
              name="subdomain"
              label="Subdomain"
              hasFeedback
              validateDebounce={300}
              rules={[
                {
                  required: true,
                  message: "Please input your subdomain!",
                },
                {
                  pattern: subdomainPattern,
                  message: "Please enter a valid subdomain!",
                },
                {
                  min: Constants.GLOBALS.MIN_SUBDOMAIN_LENGTH,
                  message: `Subdomain must be at least ${Constants.GLOBALS.MIN_SUBDOMAIN_LENGTH} characters!`,
                },
                {
                  max: Constants.GLOBALS.MAX_SUBDOMAIN_LENGTH,
                  message: "Subdomain is too long!",
                },
                {
                  required: true,
                  validator: validateSubdomain,
                },
              ]}
            >
              <Input addonBefore="https://" addonAfter=". xx.de" />
            </Form.Item>
          </Flex>
        </MyMiddleCard>
      </Form>
    </>
  );
}
