import { PlusOutlined } from "@ant-design/icons";
import { Button, Flex, Form, Input, Select } from "antd";
import HeaderBar from "core/components/Header";
import { useMessage } from "core/context/MessageContext";
import { useCreateTeamMemberMutation } from "core/services/organization";
import { Constants, EncodeStringToBase64 } from "core/utils/utils";
import { useNavigate } from "react-router-dom";
import MyMiddleCard from "shared/components/MyMiddleCard";

type FieldType = {
  firstName: string;
  lastName: string;
  email: string;
  changePasswordOnFirstLogin: boolean;
  password?: string;
  sendInvitationEmail: boolean;
};

export default function TeamCreateUser() {
  const navigate = useNavigate();
  const { success } = useMessage();

  const [reqCreateTeamMember, { isLoading }] = useCreateTeamMemberMutation();

  return (
    <>
      <HeaderBar
        theme="light"
        backTo={Constants.ROUTE_PATHS.ORGANIZATION_TEAM}
      />

      <MyMiddleCard title="Create User">
        <Form
          layout="vertical"
          requiredMark={false}
          initialValues={{
            roleId: "a1f084ad-d501-4015-b326-4c5c46fd1c5e",
          }}
          onFinish={async (values) => {
            console.log(values);

            try {
              await reqCreateTeamMember({
                firstName: values.firstName,
                lastName: values.lastName,
                email: values.email,
                roleId: values.roleId,
                password: EncodeStringToBase64(values.password),
              }).unwrap();

              success("User created successfully!");

              navigate(Constants.ROUTE_PATHS.ORGANIZATION_TEAM);
            } catch (error) {
              console.error(error);
            }
          }}
        >
          <Form.Item<FieldType>
            label="First Name"
            name="firstName"
            rules={[
              { required: true, message: "Please input first name!" },
              {
                min: Constants.GLOBALS.MIN_FIRST_NAME_LENGTH,
                message: `First name must be at least ${Constants.GLOBALS.MIN_FIRST_NAME_LENGTH} characters long!`,
              },
              {
                max: Constants.GLOBALS.MAX_FIRST_NAME_LENGTH,
                message: `First name must be at most ${Constants.GLOBALS.MAX_FIRST_NAME_LENGTH} characters long!`,
              },
            ]}
          >
            <Input
              placeholder="First Name"
              maxLength={Constants.GLOBALS.MAX_FIRST_NAME_LENGTH}
            />
          </Form.Item>

          <Form.Item<FieldType>
            label="Last Name"
            name="lastName"
            rules={[
              { required: true, message: "Please input last name!" },
              {
                min: Constants.GLOBALS.MIN_LAST_NAME_LENGTH,
                message: `Last name must be at least ${Constants.GLOBALS.MIN_LAST_NAME_LENGTH} characters long!`,
              },
              {
                max: Constants.GLOBALS.MAX_LAST_NAME_LENGTH,
                message: `Last name must be at most ${Constants.GLOBALS.MAX_LAST_NAME_LENGTH} characters long!`,
              },
            ]}
          >
            <Input
              placeholder="Last Name"
              maxLength={Constants.GLOBALS.MAX_LAST_NAME_LENGTH}
            />
          </Form.Item>

          <Form.Item<FieldType>
            label="Email"
            name="email"
            rules={[
              { required: true, message: "Please input email!", type: "email" },
            ]}
          >
            <Input placeholder="Email" />
          </Form.Item>

          <Form.Item<FieldType>
            label="Password"
            name="password"
            rules={[
              { required: true, message: "Please input password!" },
              {
                min: Constants.GLOBALS.MIN_PASSWORD_LENGTH,
                message: `Password must be at least ${Constants.GLOBALS.MIN_PASSWORD_LENGTH} characters long!`,
              },
              {
                max: Constants.GLOBALS.MAX_PASSWORD_LENGTH,
                message: `Password must be at most ${Constants.GLOBALS.MAX_PASSWORD_LENGTH} characters long!`,
              },
            ]}
          >
            <Input.Password
              placeholder="Password"
              maxLength={Constants.GLOBALS.MAX_PASSWORD_LENGTH}
            />
          </Form.Item>

          <Form.Item name="roleId" label="Role">
            <Select>
              <Select.Option value="d0f0fa0d-3f3b-438b-a76f-7febeb8aab57">
                Admin
              </Select.Option>
              <Select.Option value="b7359e12-359e-423b-b39c-f0d4069adebc">
                Moderator
              </Select.Option>
              <Select.Option value="a1f084ad-d501-4015-b326-4c5c46fd1c5e">
                User
              </Select.Option>
            </Select>
          </Form.Item>

          <Flex justify="end">
            <Button
              type="primary"
              icon={<PlusOutlined />}
              htmlType="submit"
              loading={isLoading}
            >
              Create User
            </Button>
          </Flex>
        </Form>
      </MyMiddleCard>
    </>
  );
}

/*
<Form.Item<FieldType>
      name="sendInvitationEmail"
      valuePropName="checked"
    >
      <Checkbox>Send an invitation email to the user</Checkbox>
    </Form.Item>
*/
