import {
  Avatar,
  Card,
  Descriptions,
  Flex,
  Form,
  Input,
  Typography,
} from "antd";
import HeaderBar from "../../core/components/Header";
import MyBanner from "../../shared/components/MyBanner";
import { Constants } from "core/utils/utils";
import MyMiddleCard from "shared/components/MyMiddleCard";
import Meta from "antd/es/card/Meta";
import { useGetUserProfileQuery } from "core/services/userProfile";
import { useEffect } from "react";
import {
  addWebSocketReconnectListener,
  removeWebSocketReconnectListener,
} from "core/services/websocketService";
import { useDispatch, useSelector } from "react-redux";
import {
  email,
  firstName,
  lastName,
  profilePictureUrl,
  roleId,
  setEmail,
  setFirstName,
  setLastName,
  setProfilePictureUrl,
  setRoleId,
} from "./userProfileSlice";
import { tmpRoleNames } from "features/Roles";
import MyErrorResult from "shared/components/MyResult";
import MyUpload from "shared/components/MyUpload";
import { useMessage } from "core/context/MessageContext";
import MyUserAvatar from "shared/components/MyUserAvatar";
import { setUserProfilePictureUrl } from "core/reducers/appSlice";

export default function UserProfile({ isAdmin }: { isAdmin?: boolean }) {
  const dispatch = useDispatch();
  const { success } = useMessage();

  const dataProfilePictureUrl = useSelector(profilePictureUrl);
  const dataFirstName = useSelector(firstName);
  const dataLastName = useSelector(lastName);
  const dataEmail = useSelector(email);
  const dataRoleId = useSelector(roleId);

  const { data, error, isLoading, refetch } = useGetUserProfileQuery(
    undefined,
    {
      refetchOnMountOrArgChange: true,
    }
  );

  function AdminWrapper({ children }: { children: React.ReactNode }) {
    if (!isAdmin) {
      return <>{children}</>;
    }

    return (
      <Form layout="vertical" style={{ marginTop: 24 }}>
        {children}
      </Form>
    );
  }

  function TextItem({ value, name }: { value: string; name: string }) {
    if (!isAdmin) {
      return <>{value}</>;
    }

    return (
      <Form.Item name={name} style={{ width: "100%" }} required>
        <Input defaultValue={value} />
      </Form.Item>
    );
  }

  useEffect(() => {
    if (!data) return;

    dispatch(setProfilePictureUrl(data.ProfilePictureUrl));
    dispatch(setFirstName(data.FirstName));
    dispatch(setLastName(data.LastName));
    dispatch(setEmail(data.Email));
    dispatch(setRoleId(data.RoleId));
  }, [data]);

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  return (
    <>
      <MyBanner
        title="Account Settings"
        subtitle="MANAGE"
        headerBar={<HeaderBar />}
      />

      <MyMiddleCard title="My Profile" loading={isLoading}>
        {error ? (
          <MyErrorResult />
        ) : (
          <Flex vertical gap={16} style={{ paddingBottom: 16 }}>
            <Card
              styles={{
                body: {
                  padding: 16,
                },
              }}
            >
              <Meta
                avatar={
                  <MyUpload
                    action={`/user/profile/picture`}
                    onChange={(info) => {
                      if (info.file.status === "done") {
                        success("Profile picture updated successfully");

                        dispatch(setProfilePictureUrl(info.file.response.Data));
                        dispatch(
                          setUserProfilePictureUrl(info.file.response.Data)
                        );
                      }
                    }}
                    imgCropProps={{
                      aspect: 1 / 1,
                      children: <></>,
                    }}
                  >
                    <MyUserAvatar profilePictureUrl={dataProfilePictureUrl} />
                  </MyUpload>
                }
                title={`${dataFirstName} ${dataLastName}`}
                description={tmpRoleNames[dataRoleId]}
              />
            </Card>

            <Card
              styles={{
                body: {
                  padding: 16,
                },
              }}
            >
              <AdminWrapper>
                <Descriptions
                  title="Personal Information"
                  layout="vertical"
                  items={[
                    {
                      key: "1",
                      label: "First name",
                      children: (
                        <TextItem value={dataFirstName} name="firstName" />
                      ),
                    },
                    {
                      key: "2",
                      label: "Last name",
                      children: (
                        <TextItem value={dataLastName} name="lastName" />
                      ),
                    },
                    {
                      key: "3",
                      label: "Email",
                      children: <TextItem value={dataEmail} name="email" />,
                    },
                  ]}
                />
              </AdminWrapper>
            </Card>
          </Flex>
        )}
      </MyMiddleCard>
    </>
  );
}

/*
// TODO: sessions table

  <Card
    styles={{
      body: {
        padding: 16,
      },
    }}
  >
    <Meta title="Sessions" />
    <Typography.Text>// Todoo</Typography.Text>
  </Card>
*/

function TitleText({ children }: { children: React.ReactNode }) {
  return (
    <Typography.Text style={{ fontSize: 16, opacity: 0.4 }}>
      {children}
    </Typography.Text>
  );
}

function ValueText({ children }: { children: React.ReactNode }) {
  return <Typography.Text style={{ fontSize: 16 }}>{children}</Typography.Text>;
}
